----------------------------------------------------------------------------------
-- Original Framework from: Mike Field <hamster@snap.net.nz>
-- Changes by Mario A.
-- Module Name: i2s_data_interface - Behavioral 
-- Description: Send & Receive I2S data
--              New_sample is asserted for one cycle when a new sample has been
--              received (and one transmitted)
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity i2s_data_interface is
    Port ( 
            i2s_MCLK               : in  std_logic;
            filtered_data_left     : in  std_logic_vector(23 downto 0);
            filtered_data_right    : in  std_logic_vector(23 downto 0);
            unfiltered_data_left   : out std_logic_vector(23 downto 0);
            unfiltered_data_right  : out std_logic_vector(23 downto 0);
            new_sample             : out std_logic;
            i2s_SCLK               : in  std_logic;
            i2s_DA_SDIN            : out std_logic;
            i2s_AD_SDOUT           : in  std_logic;
            i2s_LRCK               : in  std_logic
         );
end i2s_data_interface;


architecture Behavioral of i2s_data_interface is


signal SCLK_delay        : std_logic_vector(9 downto 0)   := (others => '0');
signal lr_delay          : std_logic_vector(9 downto 0)   := (others => '0');
signal sr_in             : std_logic_vector(126 downto 0) := (others => '0');
signal sr_out            : std_logic_vector(63 downto 0)  := (others => '0');
signal i2s_LRCK_last     : std_logic := '0';
signal i2s_AD_SDOUT_last : std_logic := '0';
   
   
begin
   
process(i2s_MCLK)
   begin
      -- Process to predict when the falling edge of i2s_SCLK should be
      if rising_edge(i2s_MCLK) then 
         new_sample <= '0';

         if SCLK_delay(SCLK_delay'high-1 downto SCLK_delay'high-2) = "10" then -- is there a rising edge two cycles ago? If so the data bit is validand we can capture a bit
            sr_in <= sr_in(sr_in'high-1 downto 0) & i2s_AD_SDOUT_last;
         end if;

         if SCLK_delay(1 downto 0) = "10" then  -- Was there a rising edge on SCLK 9 cycles ago? If so, this should be about the falling edge so the output can change.
            i2s_DA_SDIN <= sr_out(sr_out'high);

            -- if we are starting a new frame, then load the samples into the shift register
            if i2s_LRCK = '0' and i2s_LRCK_last = '1' then --if i2s_lRCK = '1' and i2s_LRCK_last = '0' then  --> different I2S format. Other channel first.
               unfiltered_data_left  <= sr_in(sr_in'high      downto sr_in'high - 23);
               unfiltered_data_right <= sr_in(sr_in'high - 32 downto sr_in'high - 23 - 32);
               sr_out      <= filtered_data_left & x"00" & filtered_data_right & x"00";
               new_sample <= '1';
            else
               sr_out <= sr_out(sr_out'high-1 downto 0) & '0';
            end if;
            
            i2s_LRCK_last <= i2s_LRCK; -- remember what LRCK was, for edge detection
         end if;
         
         SCLK_delay <= i2s_SCLK & SCLK_delay(SCLK_delay'high downto 1);
         i2s_AD_SDOUT_last <= i2s_AD_SDOUT;
         
      end if;
   end process;
   
end Behavioral;

